#ifndef AVR_VECFONT_CALLBACKS_H
#define AVR_VECFONT_CALLBACKS_H

#include <avr/io.h>

#include "vektor-zeichen.h"
#include "pixel.h"

// Wir beobachten folgende Gren fr vektor-zeichen.o:
//     450  ohne Definition von VECFONT_BEAM_XY und VECFONT_BEAM_SKIP
//          also unter Verwendung der Callbacks .beam_skip() und .beam_xy().
//     482  mit Definition dieser Hook-Makros als Inline-Funktionen wie unten.
//          Die Codegre steigt berraschenderweise merklich (avr-gcc 4.3.2
//          bzw. WinAVR-2009-03-13). Dies liegt an einer ungnstigen Register-
//          Allokierung, die wir ber ein asm volatile beheben.
//     416  Wir forcieren die Adresse der vecfont_t-Struktur ins Y-Register,
//          den gnstigsten Ort fr diese Adresse. Allein durch dieses
//          Inline-Assembler, das selbst KEINEN Code erzeugt, schrumpft der
//          Code des Moduls um satte 15% ! Zudem wird der Code schneller, da
//          unntige Register-Moves entfallen.

// Hook anstelle von .beam_xy
#define VECFONT_BEAM_XY(x,y)                            \
    do {                                                \
        inline_beam_xy(x,y);                            \
        asm volatile ("; RELOAD vf in %0" : "+y" (vf)); \
    } while (0)
    
// Hook anstelle von .beam_skip
#define VECFONT_BEAM_SKIP()                             \
    do {                                                \
        inline_beam_skip();                             \
        asm volatile ("; RELOAD vf in %0" : "+y" (vf)); \
    } while (0)

// Das bitadressierbare GPRIO0 (ATmega*8) dient zum Merken von Flags
#define FLAGS GPIOR0

#define FLAG_SKIP  (1 << 0)
#define FLAG_PIXEL (1 << 1)

// Die Koordinaten transportieren wir im globalen Register in R2:R3.
register upixel8_t reg_pixel asm ("r2");

// Der Wert wird ohne RAM-Zugriff zwischengespeichert fr
// baldige Auswertung
static inline void inline_beam_xy (vf_uint_t x, vf_uint_t y)
{
    reg_pixel.x = x;
    reg_pixel.y = y;
    
    FLAGS |= FLAG_PIXEL;
}

// Fr einen Positions-Sprung wird nur ein Flag gesetzt fr 
// baldige Auswertung
static inline void inline_beam_skip (void)
{
    FLAGS |= FLAG_SKIP;
}

#endif // AVR_VECFONT_CALLBACKS_H
