#include <avr/io.h>
#include <avr/pgmspace.h>

#include "vektor-zeichen.h"

// Wir brauchen hier kein #include "avr-vecfont-callbacks.h",
// denn diese Datei wird aus technischen Grnden per Kommandozeile
// includiert, damit sie auch in vektor-zeichen.c wirkt, ohne da wir
// ein #include in vektor-zeichen.c reinfrickeln mssen.

// Das Font-Objekt
vecfont_t font;

// Der Text liegt im Flash und besteht nur aus Grobuchstaben.
const char text1_p[] PROGMEM = "HALLO AVR";

int main ()
{
    while (1)
    {
        // Text initialisisern
        
        // Startposition und Text
        font.start.x = 10;
        font.start.y = 10;
        
        // Fr AVR kann der Text im Flash liegen.
        font.text = text1_p;
        // Wir mssen angeben, wie die Adresse zu 
        // interpretieren ist: SRAM oder Flash
        font.text_in_flash = 1;
        
        // Text rauspixeln.
        // Im Makefile ist VEKFONT_ZOOM=2 define'd, so da wir .zoom nicht
        // setzen mssen. Dieses Define ermglicht Optimierungen, wenn
        // zur Laufzeit nicht zwischen unterschiedlichen Zoom-Stufen
        // umgeschaltet werden mu. 
        while (1)
        {
            FLAGS &= ~FLAG_SKIP;
            FLAGS &= ~FLAG_PIXEL;
            
            // Die Callbacks sind jetzt ber Hook-Makros
            // VECFONT_BEAM_XY und VECFONT_BEAM_SKIP als Inline-Funktionen
            // definiert. Dies ermglicht weitere Optimierungen in
            // Codegre und Laufzeit.
            // Anstatt die x/y-Koordinaten auszuwerten, werden diese in einer
            // globalen Register-Variablen reg_pixel gemerkt. Dies ist sinnvoll
            // weil vecfont_draw_letters oft aufgerufen wird und somit das
            // globale Register eine gute Auslastung hat.
            // Fr einen Sprung der Ausgabeposition verwenden wir FLAGS, was
            // ein Define auf das bitadressierbare SFR GPIOR0 ist (ATmega*8).
            if (vecfont_draw_letters (&font))
                break;
                
            if (FLAGS & FLAG_SKIP)
            {
                // Sprung der Strahlposition behandeln
            }
            
            if (FLAGS & FLAG_PIXEL)
            {
                // Neue Koordinaten setzen, z.B. Ausgabe an einen D/A-Wandler
                PORTD = reg_pixel.x;
                PORTC = reg_pixel.y;
            }
        } // loop vecfont_draw_letters
    } // while (1)
    
    return 0;
}
