#include <stdio.h>
#include <stdlib.h>

#include "vektor-zeichen.h"

// Unser Vektorfont-Objekt
vecfont_t font;

// Einige Hilfsvariablen
static int char_pending;
static int px, py;
static int skip, skipped;

// Implementierung von VF_NOTIFY_CHAR.
// Wir fassen jedes Zeichen in einer eigene SVG-Gruppe zusammen
// und stellen einen kleinen Kommentar vor jede Gruppe
void notify_char (vecfont_t * vf, char c)
{
    (void) vf; // Avoid "warning: unused parameter 'vf'" from gcc
    
    // Evtl. offene Zeichen schliessen
    if (char_pending)
    {
        if (skipped)
            // Bei 2 Sprngen in Folge wurde nur 1 einzelner Pixel ausgegeben.
            // Um ihn zu sehen, mssen wir einen kleinen Strich machen. 
            // Der Pixel selbst entsteht dann durch Zusammenspiel der Attribute
            // stroke-linecap und stroke-width.
            printf ("h0.01 ");
        // PATH und G schliessen
        printf ("'/></g>\n");
        
        skipped = 0;
    }
        
    // Nur fr Zeichen != '\0' und != ' ' muss eine Gruppe erzeugt werden
    // da nur dann Pixel ausgegeben werden
    if ((char_pending = c && c != ' '))
    {
        printf ("    <!-- \"%c\" zoom=%d --> "
                "<g transform='translate(%d,%d)' stroke-width='%0.1lf'>"
                "<path d='",
                c, font.zoom, px = font.p.x, py = font.p.y, 1.0*font.zoom);
    }
    
    // Fr ' ' und '\0' nur Kommentare ausgeben
    if (c == ' ')
        printf ("    <!-- \" \" zoom=%d -->\n", font.zoom);
        
    if (c == '\0')
        printf ("    <!-- END -->\n");
}

// Callback: Der Strahl bzw. die Pixelposition macht einen Sprung
// Dieser Callback tut nichts ausser zu merken, da ein Sprung 
// stattgefunden hat.
static void beam_skip (void)
{
    skip = 1;
}

// Callback: Ein Pixel wird gesetzt
static void beam_xy (vf_uint_t x, vf_uint_t y)
{
    if (skip && skipped)
        // Bei 2 Sprngen in Folge wurde nur 1 einzelner Pixel ausgegeben.
        // Um ihn zu sehen, mssen wir einen kleinen Strich machen. 
        // Der Pixel selbst entsteht dann durch Zusammenspiel der Attribute
        // stroke-linecap und stroke-width.
        printf ("h0.01 ");

    // Bei jedem Sprung fangen wir einen neuen PATH an, um den Filzstift-
    // Effekt zu erzeugen.
    if (skip)
        printf (" '/><path d='");
        
    // Ein Skip entscheidet zwischen MoveTo (M) und LineTo (L)
    // Die Koordinaten sind absolut und beziehen sich auf die 
    // umfassende Gruppe
    printf ("%c%d,%d ", skip ? 'M' : 'L', x-px, y-py);
        
    skipped = skip;    
    skip    = 0;
}

int main (int argc, char *argv[])
{
    // Callbacks initialisieren
    font.beam_skip = beam_skip;
    font.beam_xy   = beam_xy;
    
    // Proportionalschrift
    font.proportional = 1;
    
    // Text der Kommandozeile entnehmen
    if (argc != 2)
    {
        fprintf (stderr, "\nUsage: %s \"some text\"\n\n", argv[0]);
        exit (1);
    }
    font.text = argv[1];
    
    // Startposition am oberen, linken Bildrand
    font.start.x = 10;
    font.start.y = 300;
    
    // Kopf der SVG-Datei
    printf ("<?xml version='1.0' encoding='iso-8859-1' standalone='no' ?>\n"
    //    printf ("<?xml version='1.0' encoding='UTF-8' standalone='no' ?>\n"
            "<svg\n"
            "    xmlns='http://www.w3.org/2000/svg'\n"
//            "    xmlns:xlink='http://www.w3.org/1999/xlink'\n"
            "    viewBox='0 -900 600 250'\n"
            "    width='600'\n"
            "    height='250'\n"
            "    version='1.1'>\n");
    
    // Titel
    printf ("<title>VEKFONT: \"%s\"</title>\n", font.text);
    
    // Stil fr die Linien unserer Zeichen
    printf ("<style type='text/css'>\n"
            ".letter { fill:none; "
                      "stroke:#40f; "
                      "stroke-linecap:round; "
                      "stroke-linejoin:round; "
                      "stroke-opacity:0.7; "
            "\n}\n");
    printf ("</style>\n");
    
    // Diese Gruppe umfasst alles. Wir vergrern *3 und skalieren y mit -1
    // um die Spiegelung an der x-Achse zu erreichen (ansonsten steht alles auf
    // dem Kopf, weil das SVG-Koordinatensystem den Ursprung oben links hat)
    printf ("<g transform='scale(3,-3)' class='letter' >\n");
    
    // Wir zeichenen den Text in 3 Zoom-Stufen von 1 bis 3
    for (font.zoom = 1; font.zoom <= 3; font.zoom++)
    {
        // Y-Koordinate vermindern, damit sich die Texte der
        // einzelnen Zoom-Stufen nicht berpinseln
        font.start.y -= 10 * font.zoom;
        
        // Bei jedem Aufruf wird ein Pixel gesetzt bis der Text fertig
        // ausgegeben ist. Zustzlich kann die x/y-Position einen Sprung machen.
        while (!vecfont_draw_letters (&font));
    }
    
    printf ("</g>\n");
    printf ("</svg>\n");
    
    // Fertig!
    return 0;
}
